local _G = getfenv(0);

-- Disable Blizz Casting Bars
CastingBarFrame.showCastbar = nil;
CastingBarFrame:UnregisterAllEvents();
PetCastingBarFrame.showCastbar = nil;
PetCastingBarFrame:UnregisterAllEvents();

--------------------------------------------------------------------------------------------------------
--                                           Frame Scripts                                            --
--------------------------------------------------------------------------------------------------------
function AzCastBarCast_OnLoad(self)
	self.gName = self:GetName();
	-- Init Bar
	self:EnableMouse(0);
	self.token = self.gName:gsub("AzCastBar","");
	self.unit = self.token:lower();
	tinsert(AzCastBar_Frames,self);
	-- Events
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("UNIT_SPELLCAST_START");
	self:RegisterEvent("UNIT_SPELLCAST_STOP");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED");
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
	-- Specific Frame Events
	if (self.unit == "target") then
		self:RegisterEvent("PLAYER_TARGET_CHANGED");
	elseif (self.unit == "focus") then
		self:RegisterEvent("PLAYER_TARGET_CHANGED");
		self:RegisterEvent("PLAYER_FOCUS_CHANGED");
	end
end

-- OnUpdate
function AzCastBarCast_OnUpdate(self,elapsed)
	-- Update Bar (Only player unit seems to give proper UNIT_SPELLCAST_STOP event, so start fadeout when cast completes)
	if (self.fadeTime == 0) then
		self.timeProgress = (GetTime() - self.startTime);
		if (self.timeProgress > self.castTime) then
			self.timeProgress = self.castTime;
		end
		self.timeLeft = (self.castTime - self.timeProgress);
		self.delayText = (self.castDelay == 0 and "" or ("|cffff8080"..(self.castDelay > 0 and "+" or "")..AzCastBar_FormatTime(self.castDelay).."|r  "));

		_G[self.gName.."Status"]:SetValue(self.isCast and self.timeProgress or self.timeLeft);
		_G[self.gName.."StatusRight"]:SetText(self.delayText..AzCastBar_FormatTime(self.timeLeft));

		if (self.unit == "player") and (_G[self.gName.."StatusSafezone"]:IsVisible()) then
			self.safeZonePercent = (select(3,GetNetStats()) / 1000 / self.castTime * _G[self.gName.."Status"]:GetWidth());
			_G[self.gName.."StatusSafezone"]:SetWidth(self.safeZonePercent <= 100 and self.safeZonePercent or 100);
		end

		if (self.timeLeft == 0) then
			AzCastBarCast_StartFadeOut(self);
		end
	-- Fade
	elseif (self.fadeElapsed <= self.fadeTime) then
		self.fadeElapsed = (self.fadeElapsed + elapsed);
		self:SetAlpha(self.cfg.alpha - self.fadeElapsed / self.fadeTime * self.cfg.alpha);
	else
		self.isCast, self.isChannel = nil, nil;
		self:Hide();
	end
end

--------------------------------------------------------------------------------------------------------
--                                           Event Handling                                           --
--------------------------------------------------------------------------------------------------------
function AzCastBarCast_OnEvent(self,event)
	-- End if not enabled
	if (not self.cfg or not self.cfg.enabled) then
		return;
	-- Entering World + Target/Focus Change
	elseif (event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_TARGET_CHANGED" or event == "PLAYER_FOCUS_CHANGED" or event == "AZCASTBAR_RESET") then
		if (UnitCastingInfo(self.unit)) then
			event = "UNIT_SPELLCAST_START";
			arg1 = self.unit;
		elseif (UnitChannelInfo(self.unit)) then
			event = "UNIT_SPELLCAST_CHANNEL_START";
			arg1 = self.unit;
		else
			self:Hide();
			return;
		end
	end

	-- Check Unit
	if (self.unit ~= arg1) then
		return;
	-- Hide the CastBar if another bar handles this cast (Player > Target > Focus)
	elseif (self.unit ~= "player" and UnitIsUnit(self.unit,"player")) or (self.unit == "focus" and UnitIsUnit("focus","target")) then
		self:Hide();
		return;
	-- Start
	elseif (event == "UNIT_SPELLCAST_START" or event == "UNIT_SPELLCAST_CHANNEL_START") then
		self.isCast = (event == "UNIT_SPELLCAST_START");
		self.isChannel = (event == "UNIT_SPELLCAST_CHANNEL_START");

		if (self.isCast) then
			self.spell, self.rank, self.displayName, self.icon, self.startTime, self.endTime = UnitCastingInfo(self.unit);
		else
			self.spell, self.rank, self.displayName, self.icon, self.startTime, self.endTime = UnitChannelInfo(self.unit);
		end

		self.startTime, self.endTime = (self.startTime / 1000), (self.endTime / 1000);
		self.castTime = (self.endTime - self.startTime);
		_G[self.gName.."Status"]:SetMinMaxValues(0,self.castTime);
		_G[self.gName.."Status"]:SetStatusBarColor(unpack(self.cfg.colProgress));

		if (self.unit == "player") then
			if (self.spell == "Hearthstone") and (GetBindLocation()) then
				self.rank = GetBindLocation();
			end
			if (self.cfg.safeZone) and (self.isCast) then
				_G[self.gName.."StatusSafezone"]:Show();
			end
		end

		_G[self.gName.."StatusLeft"]:SetText(self.cfg.showRank and self.rank ~= "" and self.spell.." ("..self.rank..")" or self.spell);
		_G[self.gName.."Icon"]:SetTexture(self.icon);

		self.fadeTime = 0;
		self.fadeElapsed = 0;
		self.castDelay = 0;
		self:SetAlpha(self.cfg.alpha);
		self:Show();
	-- Quit if Hidden
	elseif (not self:IsVisible()) then
		return;
	-- Failed (Happens after UNIT_SPELLCAST_STOP, so keep them here)
	elseif (event == "UNIT_SPELLCAST_FAILED") and (not self.isChannel) then
		_G[self.gName.."Status"]:SetValue(self.castTime);
		_G[self.gName.."Status"]:SetStatusBarColor(unpack(self.cfg.colFailed));
		_G[self.gName.."StatusRight"]:SetText("Failed");
		AzCastBarCast_StartFadeOut(self);
	-- Interrupted (Happens after UNIT_SPELLCAST_STOP, so keep them here)
	elseif (event == "UNIT_SPELLCAST_INTERRUPTED") and (not self.isChannel) then
		_G[self.gName.."Status"]:SetValue(self.castTime);
		_G[self.gName.."Status"]:SetStatusBarColor(unpack(self.cfg.colInterrupt));
		_G[self.gName.."StatusRight"]:SetText("Interrupted");
		AzCastBarCast_StartFadeOut(self);
	-- Quit here if no cast or channel is in progress
	elseif (self.fadeTime ~= 0) then
		return;
	-- Stop (Happens even with instant spells & abilities) (UNIT_SPELLCAST_STOP gets called right after UNIT_SPELLCAST_CHANNEL_START for some reason) (Doesn't always seem to trigger when target or focus completes cast)
	elseif (event == "UNIT_SPELLCAST_STOP" and self.isCast) or (event == "UNIT_SPELLCAST_CHANNEL_STOP" and self.isChannel) then
		_G[self.gName.."Status"]:SetValue(self.isCast and self.castTime or 0);
		AzCastBarCast_StartFadeOut(self);
	-- Cast Delayed (Az: Debug code here)
	elseif (event == "UNIT_SPELLCAST_DELAYED" or event == "UNIT_SPELLCAST_CHANNEL_UPDATE") then
		local startTime, endTime;
		if (event == "UNIT_SPELLCAST_DELAYED") then
			startTime, endTime = select(5,UnitCastingInfo(self.unit));
		else
			startTime, endTime = select(5,UnitChannelInfo(self.unit));
		end
		if (startTime and endTime) then
			local endTimeOld = self.endTime;
			self.startTime, self.endTime = (startTime / 1000), (endTime / 1000);
			self.castDelay = (self.castDelay + self.endTime - endTimeOld);
		else
			AzMsg("--- |2"..event.." are NIL|r ---");
			AzMsg("startTime = |1"..tostring(startTime).."|r, endTime = |1"..tostring(endTime).."|r.");
		end
	end
end

--------------------------------------------------------------------------------------------------------
--                                                Misc                                                --
--------------------------------------------------------------------------------------------------------

-- Start Frame FadeOut
function AzCastBarCast_StartFadeOut(self)
	if (self.fadeTime == 0) then
		if (self.unit == "player") then
			_G[self.gName.."StatusSafezone"]:Hide();
		end
		self.fadeTime = self.cfg.fadeTime;
	end
end