local _G = getfenv(0);

-- Bars
AzCastBar_Frames = {};

-- Saved Variables
AzCastBar_Config = {};
local cfg;

-- Default Config
local AZCB_DefOptions = {
	enabled = true,
	showIcon = true,
	height = 20,
	fontSize = 12,
	alpha = 1,
	backdropIndent = -3,
	texture = "Interface\\Addons\\AzCastBar\\Textures\\Waterline",
	colBackdrop = { 0.1, 0.22, 0.35, 1.0 },
	colBackground = { 1.0, 1.0, 1.0, 1.0 },
	colProgress = { 0.4, 0.6, 0.8, 1.0 },
	colFont = { 1.0, 1.0, 1.0, 1.0 },
	colFailed = { 1.0, 0.5, 0.5, 1.0 },
	colInterrupt = { 1.0, 0.75, 0.5, 1.0 },
};

--------------------------------------------------------------------------------------------------------
--                                          OnLoad & OnEvent                                          --
--------------------------------------------------------------------------------------------------------
function AzCastBar_OnLoad(self)
	-- Init Specific Mod Vars
	self.modName = "AzCastBar";
	self.modVers = GetAddOnMetadata(this.modName,"Version");
	-- Slash Commands
	SlashCmdList[self.modName] = AzCastBar_OnSlash;
	_G["SLASH_"..self.modName.."1"] = "/azcb";
	_G["SLASH_"..self.modName.."2"] = "/acb";
	-- Events
	self:RegisterEvent("VARIABLES_LOADED");
end

-- Modify All CastBars to fit the Users Settings on "VARIABLES_LOADED"
function AzCastBar_OnEvent(self,event)
	AzCastBar_CheckSettings(AzCastBar_Config);
	AzCastBar_ApplyAllSettings();
end

--------------------------------------------------------------------------------------------------------
--                                           Slash Handling                                           --
--------------------------------------------------------------------------------------------------------
function AzCastBar_OnSlash(cmd)
	-- Extract Parameters
	local param1, param2 = cmd:match("^([^%s]+)%s*(.*)$");
	param1 = (param1 and param1:lower() or cmd:lower());
	-- Options
	if (param1 == "") then
		local loaded, reason = LoadAddOn("AzCastBarOptions");
		if (loaded) then
			if (AzCastBarOptions:IsVisible()) then
				AzCastBarOptions:Hide();
			else
				AzCastBarOptions:Show();
			end
		else
			AzMsg("Could not option AzCastBar Options: |1"..tostring(reason));
		end
	-- Invalid or No Command
	else
		UpdateAddOnMemoryUsage();
		AzMsg("----- |2"..AzCastBar.modName.."|r |1"..AzCastBar.modVers.."|r ----- |1"..format("%.2f",GetAddOnMemoryUsage(AzCastBar.modName)).." |2kb|r -----");
		AzMsg("The following |2parameters|r are valid for |2/azcb|r...");
	end
end

--------------------------------------------------------------------------------------------------------
--                                           Apply Settings                                           --
--------------------------------------------------------------------------------------------------------

-- Checks if the settings exists and are correct
function AzCastBar_CheckSettings(cfgTable)
	-- Set new config if one given
	if (cfgTable) then
		cfg = cfgTable;
	end
	-- Check if each frame has settings, reset to default if they do not
	local cfgEntry;
	for _, frame in ipairs(AzCastBar_Frames) do
		if (not cfg[frame.token]) then
			cfg[frame.token] = {};
		end
		cfgEntry = cfg[frame.token];
		-- Options with different defaults
		if (cfgEntry.width == nil) then
			cfgEntry.width = (frame.token == "Mirror" and 260 or 320);
		end
		if (cfgEntry.left == nil) then
			cfgEntry.left = frame:GetLeft();
		end
		if (cfgEntry.bottom == nil) then
			cfgEntry.bottom = frame:GetBottom();
		end
		if (cfgEntry.fadeTime == nil) then
			cfgEntry.fadeTime = (frame.token == "Mirror" and 1.2 or 0.6);
		end
		if (frame.token == "Player") and (cfgEntry.safeZone == nil) then
			cfgEntry.safeZone = true;
		end
		if (frame.token ~= "Mirror") and (cfgEntry.showRank == nil) then
			cfgEntry.showRank = true;
		end
		-- Options with constant defaults
		for optionName, defOption in pairs(AZCB_DefOptions) do
			if (cfgEntry[optionName] == nil) then
				cfgEntry[optionName] = defOption;
			end
		end
	end
end

-- Apply Settings to All Bars
function AzCastBar_ApplyAllSettings()
	for _, frame in ipairs(AzCastBar_Frames) do
		AzCastBar_ApplyBarSettings(frame);
	end
end

-- Apply Settings to Given Bar
function AzCastBar_ApplyBarSettings(frame)
	if (frame.barCount) and (frame.barCount > 1) then
		for i = 1, frame.barCount do
			AzCastBar_ApplyBarSettingsSpecific(frame,i);
		end
	else
		AzCastBar_ApplyBarSettingsSpecific(frame);
	end
end

-- Apply Settings to Given Bar
function AzCastBar_ApplyBarSettingsSpecific(baseFrame,id)
	local cfgEntry = cfg[baseFrame.token];
	local frame = _G[baseFrame:GetName()..(id and id > 1 and id or "")];
	-- Give the Frame a Pointer to it's own Config table
	frame.cfg = cfgEntry;
	-- Width + Height
	frame:SetWidth(cfgEntry.width);
	frame:SetHeight(cfgEntry.height);
	-- Position
	if (not id or id == 1) then
		frame:ClearAllPoints();
		frame:SetPoint("BOTTOMLEFT",UIParent,"BOTTOMLEFT",cfgEntry.left,cfgEntry.bottom);
	end
	-- Icon
	local icon = _G[frame:GetName().."Icon"];
	if (icon) then
		if (cfgEntry.showIcon) then
			_G[frame:GetName().."Status"]:SetPoint("TOPLEFT",icon,"TOPRIGHT",3,0);
			icon:Show();
		else
			_G[frame:GetName().."Status"]:SetPoint("TOPLEFT");
			icon:Hide();
		end
		_G[frame:GetName().."Icon"]:SetWidth(cfgEntry.height);
	end
	-- Font Size
	local fontName, _, fontFlags = _G[frame:GetName().."StatusLeft"]:GetFont();
	_G[frame:GetName().."StatusLeft"]:SetFont(fontName,cfgEntry.fontSize,fontFlags);
	_G[frame:GetName().."StatusRight"]:SetFont(fontName,cfgEntry.fontSize,fontFlags);
	-- Alpha
	frame:SetAlpha(cfgEntry.alpha);
	-- Outline
	local backdrop = frame:GetBackdrop();
	backdrop.insets.left, backdrop.insets.right, backdrop.insets.top, backdrop.insets.bottom = cfgEntry.backdropIndent, cfgEntry.backdropIndent, cfgEntry.backdropIndent, cfgEntry.backdropIndent;
	frame:SetBackdrop(backdrop);
	-- Texture
	_G[frame:GetName().."StatusTexture"]:SetTexture(cfgEntry.texture);
	-- Colors
	frame:SetBackdropColor(unpack(cfgEntry.colBackdrop));
	_G[frame:GetName().."StatusBackground"]:SetVertexColor(unpack(cfgEntry.colBackground));
--	_G[frame:GetName().."StatusSafezone"]:SetVertexColor(unpack(cfgEntry.colSafezone));
	_G[frame:GetName().."Status"]:SetStatusBarColor(unpack(cfgEntry.colProgress));
	_G[frame:GetName().."StatusRight"]:SetTextColor(unpack(cfgEntry.colFont));
	_G[frame:GetName().."StatusLeft"]:SetTextColor(unpack(cfgEntry.colFont));
	-- Call the OnConfigChanged func
	if (baseFrame.OnConfigChanged) then
		baseFrame.OnConfigChanged();
	end
end

--------------------------------------------------------------------------------------------------------
--                                                Misc                                                --
--------------------------------------------------------------------------------------------------------

-- Format time as it is shown on the CastBar
function AzCastBar_FormatTime(sec)
	if (sec <= 60) then
		return ("%.1f"):format(sec);
	else
		return ("%d:%.2d"):format(sec/60,mod(sec,60));
	end
end

--------------------------------------------------------------------------------------------------------
--                                Global Chat Message Function (Rev 3)                                --
--------------------------------------------------------------------------------------------------------
if (not AZMSG_REV or AZMSG_REV < 3) then
	AZMSG_REV = 3;
	function AzMsg(text)
		DEFAULT_CHAT_FRAME:AddMessage(tostring(text):gsub("|1","|cffffff80"):gsub("|2","|cffffffff"),128/255,192/255,255/255);
	end
end