local _G = getfenv(0);

-- Icons
local AZCB_TimerIcons = {
	[0] = "Interface\\Icons\\Spell_Shadow_SoulLeech_2",
	["BREATH"] = "Interface\\Icons\\Spell_Shadow_DemonBreath",
	["FEIGNDEATH"] = "Interface\\Icons\\Ability_Rogue_FeignDeath",
};

-- Disable Blizz Mirror Bars
UIParent:UnregisterEvent("MIRROR_TIMER_START");
MirrorTimer1:UnregisterAllEvents();
MirrorTimer2:UnregisterAllEvents();
MirrorTimer3:UnregisterAllEvents();

--------------------------------------------------------------------------------------------------------
--                                           Frame Scripts                                            --
--------------------------------------------------------------------------------------------------------
function AzCastBarMirror_OnLoad(self)
	-- Init Bar
	self:EnableMouse(0);
	self.token = "Mirror";
	if (self:GetID() == 1) then
		self.barCount = 3;
		tinsert(AzCastBar_Frames,self);
	end
	-- Events
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("MIRROR_TIMER_START");
	self:RegisterEvent("MIRROR_TIMER_STOP");
	self:RegisterEvent("MIRROR_TIMER_PAUSE");
end

function AzCastBarMirror_OnUpdate(self,elapsed)
	-- Update Bar (Az: Fadeout should be called on MIRROR_TIMER_STOP, but we might need to force it here)
	if (self.fadeTime == 0) then
		self.timeProgress = (GetMirrorTimerProgress(self.timerId) / 1000);
		self.timeLeft = (self.timeProgress < 0 and 0) or (self.timeProgress < self.maxValue and self.timeProgress) or (self.maxValue);
		_G[self:GetName().."Status"]:SetValue(self.timeLeft);
		_G[self:GetName().."StatusRight"]:SetText(AzCastBar_FormatTime(self.timeLeft));
		--if (self.timeLeft == 0) then
		--	self.timerId = nil;
		--	self.fadeTime = self.cfg.fadeTime;
		--end
	-- Fade
	elseif (self.fadeElapsed < self.fadeTime) then
		self.fadeElapsed = (self.fadeElapsed + arg1);
		self:SetAlpha(self.cfg.alpha - self.fadeElapsed / self.fadeTime * self.cfg.alpha);
	else
		self:Hide();
	end
end

--------------------------------------------------------------------------------------------------------
--                                           Event Handling                                           --
--------------------------------------------------------------------------------------------------------
function AzCastBarMirror_OnEvent(self,event)
	-- End if not enabled
	if (not self.cfg.enabled) then
		return;
	-- Check for Timers on login
	elseif (event == "PLAYER_ENTERING_WORLD" or event == "AZCASTBAR_RESET") then
		for i = 1, 3 do
			arg1, arg2, arg3, arg4, arg5, arg6 = GetMirrorTimerInfo(i);
			if (arg1) and (arg1 ~= "UNKNOWN") then
				event = "MIRROR_TIMER_START";
				break;
			end
		end
		if (not arg1 or arg1 == "UNKNOWN") then
			return;
		end
	end


	-- End if not our timerId
	if (self.timerId) and (self.timerId ~= arg1) then
		return;
	-- Start
	elseif (event == "MIRROR_TIMER_START") then
		for i = 1, 3 do
			if (self:GetID() ~= i) and (_G["AzCastBarMirror"..(i > 1 and i or "")].timerId == arg1) then
				return;
			end
		end

		self.timerId = arg1;
		--self.value = arg2;
		self.maxValue = (arg3 / 1000);
		self.label = arg6;
		--self.scale = arg4;
		--self.paused = arg5;

		_G[self:GetName().."Status"]:SetMinMaxValues(0,self.maxValue);
		_G[self:GetName().."Status"]:SetStatusBarColor(unpack(self.cfg.colProgress));

		_G[self:GetName().."StatusLeft"]:SetText(self.label);
		_G[self:GetName().."Icon"]:SetTexture(AZCB_TimerIcons[self.timerId] or AZCB_TimerIcons[0]);

		self.fadeTime = 0;
		self.fadeElapsed = 0;
		self:SetAlpha(self.cfg.alpha);
		self:Show();
	-- Stop
	elseif (event == "MIRROR_TIMER_STOP") then
		if (self.timerId == arg1) and (self.fadeTime == 0) then
			self.timerId = nil;
			self.fadeTime = self.cfg.fadeTime;
		end
	end
end