local _G = getfenv(0);

-- General
local BASE_ATTACK = "^.+ claims the (.+)!  If left unchallenged, the (.+) will control it"; -- Happens when base was uncontrolled, Neutral to Horde or Alliance

-- Warsong Gulch
local WSG_GAME_START = "^The battle for (.+) begins in (%d+) (.+)%.";

-- Alterac Valley
local AV_GAME_START = "^(%d+) (.+) until the battle for (.+) begins%.";
local AV_CAPTURE_TIME = (60 * 4);
local AV_CAPTURE_TIME_SNOWFALL = (60 * 5);
local AV_GY_ATTACK =  "^The (.+) is under attack!  If left unchecked, the (.+) will .+ it!"; -- Last .+ is either "destroy" or "control"
local AV_TOWER_ATTACK = "^(.+) is under attack!  If left unchecked, the (.+) will destroy it!"; -- Sometimes starts with "The" at the start, but then it will be cought by the AV_GY_ATTACK
local AV_DEFEND = "^The (.+) was taken by the (.+)!";
local AV_DEFEND2 = "^(.+) was taken by the (.+)!";

-- Arathi Basin
local AB_GAME_START ="^The Battle for (.+) will begin in (%d+) (.+)";
local AB_CAPTURE_TIME = 60;
local AB_BASE_ATTACK = "^.+ has assaulted the (.+)!";
local AB_BASE_DEFEND = "^.+ has defended the (.+)!";

-- Eye of the Storm
local EOTS_GAME_START = "^The battle begins in (%d+) (.+)!";

-- Icons
local BG_ICONS = {
	["Alliance"] = "Interface\\Icons\\Inv_Misc_SummerFest_BrazierBlue",
	["Horde"] = "Interface\\Icons\\Inv_Misc_SummerFest_BrazierRed",
};

local timers = {};

--------------------------------------------------------------------------------------------------------
--                                           Frame Scripts                                            --
--------------------------------------------------------------------------------------------------------
function BGTimerMain_OnLoad(self)
	-- Events
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL");
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE");
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE");
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
	-- Init AzCastBar Entry
	BGTimers.token = "BGTimers";
	BGTimers.barCount = 1;
	BGTimers.OnConfigChanged = BGTimers_ConfigChanged;
	BGTimers.OnEditMode = BGTimers_UpdateTimers;
	tinsert(AzCastBar_Frames,BGTimers);
end

--------------------------------------------------------------------------------------------------------
--                                           Event Handling                                           --
--------------------------------------------------------------------------------------------------------
function BGTimerMain_OnEvent(self,event)
	-- End if disabled
	if (not BGTimers.cfg.enabled) then
		return;
	-- Hide all bars upon zoning.
	elseif (event == "PLAYER_ENTERING_WORLD") then
		for index, table in ipairs(timers) do
			table.fadeTime = BGTimers.cfg.fadeTime;
		end
	-- End if not in a BG
	elseif (select(2,IsInInstance()) ~= "pvp") then
		return;
	-- Yells
	elseif (event == "CHAT_MSG_MONSTER_YELL") and (arg2 == "Herald") then
		-- attack (gy + tower)
		local place, faction = arg1:match(AV_GY_ATTACK);
		if (not place) then
			place, faction = arg1:match(AV_TOWER_ATTACK);
		end
		if (place) then
			BGTimers_AddTimer(place,BGTimers_GetDuration(place),faction);
			return;
		end
		-- defend
		place, faction = arg1:match(AV_DEFEND);
		if (not place) then
			place, faction = arg1:match(AV_DEFEND2);
		end
		if (place) then
			BGTimers_RemoveTimer(place);
			return;
		end
	-- Alliance / Horde
	elseif (event == "CHAT_MSG_BG_SYSTEM_HORDE" or event == "CHAT_MSG_BG_SYSTEM_ALLIANCE") then
		-- attack
		local base, faction = arg1:match(BASE_ATTACK);
		if (not base) then
			base = arg1:match(AB_BASE_ATTACK);
		end
		if (base) then
			faction = (event == "CHAT_MSG_BG_SYSTEM_HORDE" and "Horde" or "Alliance");
			BGTimers_AddTimer(base,BGTimers_GetDuration(base),faction);
			return;
		end
		-- defend
		base = arg1:match(AB_BASE_DEFEND);
		if (base) then
			BGTimers_RemoveTimer(base);
			return;
		end
	-- Neutral
	elseif (event == "CHAT_MSG_BG_SYSTEM_NEUTRAL") then
		local bg, time, units = arg1:match(WSG_GAME_START);
		if (not time) then
			time, units, bg = arg1:match(AV_GAME_START);
			if (not time) then
				bg, time, units = arg1:match(AB_GAME_START);
				if (not time) then
					time, units = arg1:match(EOTS_GAME_START);
				end
			end
		end
		if (time) then
			BGTimers_AddTimer((bg or "The Game").." Begins",units:match("minutes?") and time * 60 or time);
			return;
		end
	end
end


--------------------------------------------------------------------------------------------------------
--                                            Bar OnUpdate                                            --
--------------------------------------------------------------------------------------------------------

function BGTimers_OnUpdate(self,elapsed)
	local timer = self.timer;
	if (timer.fadeTime == -1) then
		self.timeProgress = (GetTime() - timer.startTime);
		self.timeLeft = max(0,timer.seconds - self.timeProgress);
		_G[self:GetName().."Status"]:SetValue(self.timeProgress);
		_G[self:GetName().."StatusRight"]:SetText(AzCastBar_FormatTime(self.timeLeft));
		if (self.timeLeft == 0) then
			timer.fadeTime = self.cfg.fadeTime;
		end
	elseif (timer.fadeElapsed < timer.fadeTime) then
		timer.fadeElapsed = (timer.fadeElapsed + elapsed);
		self:SetAlpha(1 - timer.fadeElapsed / timer.fadeTime);
	else
		self:Hide();
		BGTimers_RemoveTimer(timer.label);
		BGTimers_UpdateTimers();
	end
end

--------------------------------------------------------------------------------------------------------
--                                                Code                                                --
--------------------------------------------------------------------------------------------------------

-- Add Timer
function BGTimers_AddTimer(label,seconds,faction)
	for index, table in ipairs(timers) do
		if (table.label == label) then
			tremove(timers,index);
			break;
		end
	end
	tinsert(timers,{ label = label, seconds = seconds, faction = faction, startTime = GetTime(), fadeTime = -1, fadeElapsed = 0 });
	table.sort(timers,function(a,b) return (a.startTime + a.seconds) < (b.startTime + b.seconds) end);
	BGTimers_UpdateTimers();
end

-- Remove Timer with given label
function BGTimers_RemoveTimer(label)
	for index, table in ipairs(timers) do
		if (table.label == label) then
			tremove(timers,index);
			BGTimers_UpdateTimers();
			break;
		end
	end
end

-- Update Timers
function BGTimers_UpdateTimers()
	-- Loop Timers and update bars
	local bar;
	for index, table in ipairs(timers) do
		bar = _G["BGTimers"..(index > 1 and index or "")];
		if (not bar) then
			BGTimers.barCount = index;
			bar = CreateFrame("Frame","BGTimers"..index,UIParent,"BGTimerBarTemplate");
			AzCastBar_ApplyBarSettingsSpecific(BGTimers,index);
			bar:SetPoint("TOP","BGTimers"..(index - 1 > 1 and index - 1 or ""),"BOTTOM",0,bar.cfg.backdropIndent*2-1);
		end

		bar.timer = table;

		_G[bar:GetName().."Status"]:SetMinMaxValues(0,table.seconds);
		if (table.faction == "Horde") then
			_G[bar:GetName().."Status"]:SetStatusBarColor(unpack(bar.cfg.colFailed));
		else
			_G[bar:GetName().."Status"]:SetStatusBarColor(unpack(bar.cfg.colProgress));
		end

		_G[bar:GetName().."StatusLeft"]:SetText(BGTimers_TitleCase(table.label));
		_G[bar:GetName().."Icon"]:SetTexture(table.faction and BG_ICONS[table.faction] or "Interface\\Icons\\Spell_Nature_UnrelentingStorm");

		if (table.fadeTime == -1) then
			bar:SetAlpha(bar.cfg.alpha);
		end
		bar:Show();
	end
	-- Hide all other frames
	for i = #timers + 1, BGTimers.barCount do
		_G["BGTimers"..(i > 1 and i or "")]:Hide();
	end
end

-- Get time duration
function BGTimers_GetDuration(base)
	if (base == "Snowfall graveyard") then
		return AV_CAPTURE_TIME_SNOWFALL;
	elseif (GetRealZoneText() == "Alterac Valley") then
		return AV_CAPTURE_TIME;
	else
		return AB_CAPTURE_TIME;
	end
end

-- Config Change
function BGTimers_ConfigChanged()
	local bar;
	for i = 2, BGTimers.barCount do
		bar = _G["BGTimers"..(i > 1 and i or "")];
		bar:ClearAllPoints();
		bar:SetPoint("TOP","BGTimers"..(i - 1 > 1 and i - 1 or ""),"BOTTOM",0,bar.cfg.backdropIndent*2-1);
	end
	--BGTimers_UpdateTimers();
end

-- Title Case
function BGTimers_TitleCase(string)
	return string:gsub("^(.)",strupper):gsub("( .)",strupper);
end